#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>

#include "BPHEME.h"

#ifndef L
#define L 256
#endif


#define JMAX 300

double BPHEME::rtnewt( void (*funcd)(double,double,double*,double*),double x1,double x2,double xacc,double mu)
{
	int j;
	double f,df,dx,rtn;

	rtn = 0.5*(x1+x2); /* initial guess */
	for(j=1;j<=JMAX;j++)
	{
		(*funcd)(rtn,mu,&f,&df);
		dx = f/df;
		rtn -= dx;
		if ( (x1-rtn)*(rtn-x2) < 0. ) 
		{
			fprintf(stderr,"Jumped out of brackets in rtnewt (%lf)\n",rtn);
			exit(1);
		}
		if ( fabs(dx) < xacc )
		{
			fprintf(stdout,"iter = %d\n",j); 
			return rtn;
		}
	}
	fprintf(stderr,"Maximum number of iterions exceeded in rtnewt\n");
	return rtn;
}

void flambda2(double x,double mu,double *f, double *df)
{
	*f  = ( 1. - mu ) * x * exp ( x ) - exp(x) + 1. + mu * x ;
	*df = ( 1. - mu ) * ( exp ( x ) + x * exp ( x ) ) - exp ( x ) + mu;
}

double BPHEME::getlambda2(double mean)
{
	if (mean == 0.5)
		return 0.;
	if ( mean > 0.5 )
		return rtnewt(flambda2,   0.,200.,0.000001,mean);
	else
		return rtnewt(flambda2,-200.,  0.,0.000001,mean);
}

BPHEME::BPHEME(char *file,int bEHE) : Equalization(file,bEHE,AL_BPHEME)
{
//	m_pZ  = (double*)calloc(L,sizeof(double));
}

BPHEME::~BPHEME()
{
}


void BPHEME::Equalizing()
{
	double mean,lambda2;
	register int l,k;

	/* Mean calculation */
	mean = 0.;
	for ( l = 0 ; l < L ; l++ )
		mean += l * m_pH[l];
	mean /= m_iPixels;
	fprintf(stdout,"mean = %lf (%10.8lf)\n",mean,mean/(L-1.));

	lambda2 = getlambda2(mean/(L-1.));
	fprintf(stdout,"lambda2 = %lf\n",lambda2);

	/* cumumative function computation - T(r)*/
	m_pC[0] = (double) m_pH[0] / m_iPixels;
	for ( l = 1 ; l < L ; l++ )
	{
		m_pC[l] = (double) m_pH[l] / m_iPixels;
		m_pC[l] += m_pC[l-1];
	}

	/* T[z] */
	for ( l = 0 ; l < L ; l++ )
		m_pZ[l] = (double) ( exp( lambda2 * l/(L-1.) ) - 1. ) / ( exp( lambda2 ) - 1. ) ;

	if ( mean/(L-1.) == 0.5)
	{
		for ( l = 0 ; l < L ; l++ )
			m_pNL[l] = l;
	}
	else
	{
		/* Histogram specification - equalization */
		for ( l = 0 ; l < L ; l++ )
		{
			for (m_pNL[l] = 0 , k = 1; k < L ; k++)
				if ( ( fabs(m_pC[l] - m_pZ[m_pNL[l]]) > fabs(m_pC[l] - m_pZ[k]) ) ) 
					m_pNL[l] = k;
//			fprintf(stdout,"[%d] = %d\n",l,m_pNL[l]);
		}
	}

	Equalization::Equalizing();
}

const char* BPHEME::GetAlgorithm()
{
	return "BPHEME";
}
