// DIB.cpp : implementation file
//

/* 
#pragma pack(1)
foi comentado porque transforma o tamanho de jpeg_decompress_struct de 464 para 460, deixando incompativel com jpeglib.h
para nao comprometer o resto, #pragma pack(1) foi adicionado a cada struct do DIB.h
Andre, 11-04-2005
*/

#include <math.h> 
#include <string.h>

#include "DIB.h"

/////////////////////////////////////////////////////////////////////////////
// CDIB class

CDIB::CDIB()
{
	ClearAttributes();	// Limpa os Atributos da Classe
}

CDIB::CDIB(CDIB &New)
{
	ClearAttributes();	// Limpa os Atributos da Classe

	ReplaceImage(New.CopyImage(m_nImage),m_nImage);
}

CDIB::CDIB(CDIB *New)
{
	ClearAttributes();	// Limpa os Atributos da Classe

	ReplaceImage(New->CopyImage(m_nImage),m_nImage);
}

CDIB::~CDIB()
{
	if (m_pImage!= NULL)
	{
		free(m_pImage);
		m_pImage = NULL;
		m_nImage = -1;
	}

	if (m_palDIB != NULL)
		delete m_palDIB;
}

/////////////////////////////////////////////////////////////////////////////
// CDIB message handlers

void CDIB::ClearAttributes()
{
	m_palDIB		= NULL;			// paleta do bitmap
	m_pImage		= NULL;			// ponteiro para o bitmap
	m_pBits			= NULL;			// ponteiro para os bits do bitmap
	m_InitSCorner	= FALSE;		// qual a primeira linha do Bitmap : V - superior ; F - inferior
	m_Height		= 0;			// altura do bitmap
	m_Width			= 0;			// largura do bitmap
	m_WidthBytes	= 0;			// numero de bytes por linha do bitmap
	m_BitsPPixel	= 0;			// numero de bits por pixel
	m_XPxls			= 0;			// numero de pixels por metro na vertical
	m_YPxls			= 0;			// numero de pixels por metro na horizontal
}

void CDIB::AssertValidDib() const
{
	ASSERT(m_pImage);
	ASSERT(m_nImage!=-1);
	ASSERT(m_pBits);
}

CDIB CDIB::operator+(CDIB &op)
{
	CDIB res(this);
	UINT lin, col;
	int sub;
	if(res != op)
		return res;
	switch(GetBitsPerPixel())
	{
	case 1 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelBW(col, lin) + op.GetPixelBW(col, lin);
				if(sub < 0)
					res.SetPixelBW(col, lin, 0);
				else if(sub > 255)
					res.SetPixelBW(col, lin, 1);
				else
					res.SetPixelBW(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 4 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG4(col, lin) + op.GetPixelG4(col, lin);
				if(sub < 0)
					res.SetPixelG4(col, lin, 0);
				else if(sub > 15)
					res.SetPixelG4(col, lin, 15);
				else
					res.SetPixelG4(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 8 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG8(col, lin) + op.GetPixelG8(col, lin);
				if(sub < 0)
					res.SetPixelG8(col, lin, 0);
				else if(sub > 255)
					res.SetPixelG8(col, lin, 255);
				else
					res.SetPixelG8(col, lin, (BYTE)sub);
			}  
		}
		break;
	}
	return res;
}

CDIB CDIB::operator+(CDIB *op)
{
	CDIB res(this);
	UINT lin, col;
	int sub;
	if(res != op)
		return res;
	switch(GetBitsPerPixel())
	{
	case 1 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelBW(col, lin) + op->GetPixelBW(col, lin);
				if(sub < 0)
					res.SetPixelBW(col, lin, 0);
				else if(sub > 255)
					res.SetPixelBW(col, lin, 1);
				else
					res.SetPixelBW(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 4 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG4(col, lin) + op->GetPixelG4(col, lin);
				if(sub < 0)
					res.SetPixelG4(col, lin, 0);
				else if(sub > 15)
					res.SetPixelG4(col, lin, 15);
				else
					res.SetPixelG4(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 8 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG8(col, lin) + op->GetPixelG8(col, lin);
				if(sub < 0)
					res.SetPixelG8(col, lin, 0);
				else if(sub > 255)
					res.SetPixelG8(col, lin, 255);
				else
					res.SetPixelG8(col, lin, (BYTE)sub);
			}  
		}
		break;
	}
	return res;
}

CDIB CDIB::operator-(CDIB &op)
{
	CDIB res(this);
	UINT lin, col;
	int sub;
	if(res != op)
		return res;
	switch(GetBitsPerPixel())
	{
	case 1 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelBW(col, lin) - op.GetPixelBW(col, lin);
				if(sub < 0)
					res.SetPixelBW(col, lin, 0);
				else if(sub > 255)
					res.SetPixelBW(col, lin, 1);
				else
					res.SetPixelBW(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 4 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG4(col, lin) - op.GetPixelG4(col, lin);
				if(sub < 0)
					res.SetPixelG4(col, lin, 0);
				else if(sub > 15)
					res.SetPixelG4(col, lin, 15);
				else
					res.SetPixelG4(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 8 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG8(col, lin) - op.GetPixelG8(col, lin);
				if(sub < 0)
					res.SetPixelG8(col, lin, 0);
				else if(sub > 255)
					res.SetPixelG8(col, lin, 255);
				else
					res.SetPixelG8(col, lin, (BYTE)sub);
			}  
		}
		break;
	}
	return res;
}

CDIB CDIB::operator-(CDIB *op)
{
	CDIB res(this);
	UINT lin, col;
	int sub;
	if(res != op)
		return res;
	switch(GetBitsPerPixel())
	{
	case 1 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelBW(col, lin) - op->GetPixelBW(col, lin);
				if(sub < 0)
					res.SetPixelBW(col, lin, 0);
				else if(sub > 255)
					res.SetPixelBW(col, lin, 1);
				else
					res.SetPixelBW(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 4 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG4(col, lin) - op->GetPixelG4(col, lin);
				if(sub < 0)
					res.SetPixelG4(col, lin, 0);
				else if(sub > 15)
					res.SetPixelG4(col, lin, 15);
				else
					res.SetPixelG4(col, lin, (BYTE)sub);
			}  
		}
		break;
	case 8 :
		for(lin = 0; lin < GetImageHeight(); lin++)
		{
			for(col = 0; col < GetImageWidth(); col++)
			{
				sub = GetPixelG8(col, lin) - op->GetPixelG8(col, lin);
				if(sub < 0)
					res.SetPixelG8(col, lin, 0);
				else if(sub > 255)
					res.SetPixelG8(col, lin, 255);
				else
					res.SetPixelG8(col, lin, (BYTE)sub);
			}  
		}
		break;
	}
	return res;
}

CDIB CDIB::operator=(CDIB &op)
{
	ReplaceImage(op.CopyImage(m_nImage),m_nImage);
	return *this;
}

CDIB CDIB::operator=(CDIB *op)
{
	ReplaceImage(op->CopyImage(m_nImage),m_nImage);
	return *this;
}

BOOL CDIB::operator!=(CDIB &op)
{
	if(m_Width == op.GetImageWidth() || m_Height == op.GetImageHeight())
		return FALSE;
	return TRUE;
}

BOOL CDIB::operator!=(CDIB *op)
{
	if(m_Width == op->GetImageWidth() || m_Height == op->GetImageHeight())
		return FALSE;
	return TRUE;
}

BOOL CDIB::operator==(CDIB &op)
{
	if(!(m_Width == op.GetImageWidth() && m_Height == op.GetImageHeight()))
		return TRUE;
	return FALSE;
}

BOOL CDIB::operator==(CDIB *op)
{
	if(m_Width == op->GetImageWidth() && m_Height == op->GetImageHeight())
		return TRUE;
	return FALSE;
}

// Abre jpg ou bmp
BOOL CDIB::AbrirArquivo(char *nomeArq)
{
	char *ext;

	ext = strrchr(nomeArq, '.') + 1;
	if (ext == NULL)
	{
		printf("Erro: o arquivo de entrada precisa ter uma extensao!\n");
		exit(1);	
	}
	
//	if (!strcmp(ext, "jpg"))
//		return LoadJPGFromFile(nomeArq);
//	else
		if (!strcmp(ext, "bmp"))
		return LoadBMPFromFile(nomeArq);
	else
	{
		printf("Erro: extensao invalida para arquivo %s\n", nomeArq);
		exit(0);
	}
}

/*
BOOL CDIB::LoadJPGFromFile(PCTSTR pszPathName) 
{	
	FILE * arquivo;

	if ((arquivo=fopen(pszPathName, "r")) == NULL)
	{
		fprintf(stderr,"LoadJPGFromFile: Erro ao abrir arquivo via fopen\n");
		return FALSE;
	}
	
	LoadJPG(arquivo);
	
	if (fclose(arquivo) == EOF)
	{
		fprintf(stderr,"LoadJPGFromFile: Erro ao fechar arquivo!\n");
		return FALSE;
	}

	//InitDIBData(); Ja eh feito em LoadJPG

	if (m_pImage == NULL)
	{
		fprintf(stderr,"LoadJPGFromFile: Nao foi possivel carregar JPG!\n");
		return FALSE;
	}

	return TRUE;
}
*/

BOOL CDIB::LoadBMPFromFile(PCTSTR pszPathName)
{
	FILE* fdFile;

	if ( (fdFile=fopen(pszPathName, "rb")) == NULL)
	{
		fprintf(stderr,"LoadBMPFromFile: Erro ao abrir arquivo!\n");
		return FALSE;
	}

	LoadBMP(fdFile);

	if ( fclose(fdFile) != 0)
	{
		fprintf(stderr,"LoadBMPFromFile: Erro ao fechar arquivo!\n");
		return FALSE;
	}

	InitDIBData();

	if (m_pImage == NULL)
	{
		fprintf(stderr,"LoadBMPFromFile: Nao foi possivel carregar o BMP!\n");
		return FALSE;
	}
	return TRUE;
}	


/*
BOOL CDIB::LoadTIFFFromFile(PSTR lpszPathName)//CFile& file)
{
	TIFF* tif;
	unsigned int   Length;			// tiff uint32 
	unsigned int   Width;			// tiff uint32
	unsigned short BitsPerSample;	// tiff uint16
	unsigned short SamplesPerPixel; // tiff uint16
	unsigned short Photometric;		// tiff uint16
	BYTE *buf = NULL;			// tiff tdata_t
	BYTE *aux = NULL;

	BeginWaitCursor();
	DeleteContents();

	tif = TIFFOpen(lpszPathName, "r");
	if(tif == NULL)
		return FALSE;

	if(!TIFFGetField(tif, TIFFTAG_IMAGELENGTH    ,&Length ))
		return FALSE;
	if(!TIFFGetField(tif, TIFFTAG_IMAGEWIDTH     ,&Width  ))
		return FALSE;

	buf = new BYTE[Length * Width * 4 ];
	TIFFReadRGBAImage(tif, Width, Length, (unsigned long*)buf);
	
	ReplaceHandleImage(CreateHandleImage(Width, Length, 24));
	UINT cont = Width * Length;
	UINT contL = 0;
	aux = buf;
	while(cont > 0)
	{
		*(lpBits+2) = *aux++;
		*(lpBits+1) = *aux++;
		*lpBits = *aux++;
		lpBits+=3;
		aux++;
		cont--;
		contL+=3;
		if(contL == Width*3)
		{
			while(contL < WIDTHBYTES(Width*24))
			{
				contL++;
				lpBits++;
			}
			contL=0;
		}
	}
	
	delete []buf;

	InitDIBData();
	ClearRegion();

	if (GetHandleImage() == NULL)
	{
		// may not be DIB format
		CString strMsg;
		strMsg.LoadString(IDS_CANNOT_LOAD_DIB);
		MessageBox(NULL, strMsg, NULL, MB_ICONINFORMATION | MB_OK);
//		EndWaitCursor();
		return FALSE;
	}

	if(!TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &SamplesPerPixel))
		return FALSE;
	if(!TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &Photometric))
	{
		switch(SamplesPerPixel)
		{
		case 1:
			Photometric = PHOTOMETRIC_MINISBLACK;
			break;
		case 3:
		case 4:
			Photometric = PHOTOMETRIC_RGB;
			break;
		default :
			return FALSE;
		}
	}
	switch(Photometric)
	{
	case PHOTOMETRIC_MINISWHITE:
	case PHOTOMETRIC_MINISBLACK:
		switch (SamplesPerPixel) 
		{
		case 1:
			TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &BitsPerSample);
			CConversao *pConversao;
			switch(BitsPerSample)
			{
			case 1:
				pConversao->ConversaoBinario(this);
				break;
			case 4:
				pConversao->ConversaoCinza16(this);
				break;
			case 8:
				pConversao->ConversaoCinza256(this);
				break;
			}
			break;
		case 3: case 4:
			break;
		default:
			return FALSE;
		}
		break;
	case PHOTOMETRIC_PALETTE:
	case PHOTOMETRIC_RGB:
	case PHOTOMETRIC_YCBCR:
		break;
	default :
		return FALSE;
	}

	TIFFClose(tif);
//	SetPathName(lpszPathName);
//	SetModifiedFlag(FALSE);     // start off with unmodified
//	EndWaitCursor();
	return TRUE;
}
*/

/*
BOOL CDIB::LoadGIFFromFile(PSTR lpszPathName)			// Abre um arquivo do tipo GIF (.gif) com o nome indicado (caminho completo)
{
	FILE *arquivo;
	DWORD tam;
	FBM *image = new FBM;
	char identificador[3];//3 bytes de identificao do arquivo


	image->cm = NULL;
	image->bm = NULL;

	if( (arquivo = fopen( lpszPathName, "r+b" )) == NULL )
	{
		TRACE("Erro ao abrir novo arquivo GIF do tipo FILE*");
		return FALSE;
	}

	//copiar os 3 primeiros bytes do arquivo para identificador

	if(fseek( arquivo, 0L, SEEK_END))
	{
		TRACE( "Fseek failed" );
		return FALSE;
	}
	tam = ftell(arquivo);
	
	if(fseek( arquivo, 0L, SEEK_SET))
	{
		TRACE( "Fseek failed" );
		return FALSE;
	}

	UINT nBytesRead = fread(identificador, 3, 1, arquivo);

	if(fseek( arquivo, 0L, SEEK_SET))
	{
		TRACE( "Fseek failed" );
		return FALSE;
	}

  	if ((identificador[0]=='G') && (identificador[1]=='I') && (identificador[2]=='F'))
	{
		if(!read_gif((::FBM*)image, arquivo, NULL, 0))
		{
			TRACE("Falha na leitura do arquivo gif.");
			free_fbm(image);
			delete image;
			return FALSE;
		}
		ConverterGIFParaBMP(image);

		InitDIBData();
		ClearRegion();

		free_fbm(image);
		delete image;
		return TRUE;
	}
	return FALSE;
}
*/

BOOL CDIB::SaveBMPFromFile(PCTSTR pszPathName)
{	
	FILE* fdFile;

#ifdef LINUX
	chmod(pszPathName,S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
#endif	
	if ( (fdFile=fopen(pszPathName, "wb+")) == NULL)
	{
		fprintf(stderr,"SaveBMPFromFile: Erro ao abrir arquivo para gravacao via open()\n");
		return FALSE;
	}
#ifdef LINUX
	fchmod(fdFile,S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
#endif
	BOOL bSuccess = SaveBMP(fdFile);

       	if ( fclose(fdFile) != 0)
	{
		fprintf(stderr,"SaveBMPFromFile: Erro ao fechar arquivo!\n");
		return FALSE;
	}

	if (!bSuccess)
		fprintf(stderr,"SaveBMPFromFile: No foi possvel Salvar arquivo!\n");

	return bSuccess;
}

/*
BOOL CDIB::SaveTIFFFromFile(PSTR lpszPathName)
{
	
	TIFF* tif;
//	unsigned int Length;			// tiff uint32
//	unsigned int Width;				// tiff uint32
//	unsigned short BitsPerSample;	// tiff uint16
//	unsigned short SamplesPerPixel; // tiff uint16
	BYTE *buf = NULL;				// tiff tdata_t
	BYTE *aux = NULL;
	UINT y;

//	BeginWaitCursor();

	tif = TIFFOpen(lpszPathName, "w");
	if(tif == NULL)
		return FALSE;

	if(!TIFFSetField(tif, TIFFTAG_IMAGELENGTH, GetImageHeight()))
		return FALSE;

	if(!TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, GetImageWidth()))
		return FALSE;

	if(!TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_LZW))
		return FALSE;

	if(!TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG))
		return FALSE;
	
	if(!TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK))
		return FALSE;

	switch(GetBitsPerPixel())
	{
	case 1:
		if(!TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 1))
			return FALSE;
		if(!TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1))
			return FALSE;
		aux = (BYTE*)lpBits + m_Size;
		for(y = 0; y < GetImageHeight(); y++)
		{
			aux -= WIDTHBYTES(GetImageWidth() * GetBitsPerPixel());
			TIFFWriteScanline(tif, aux, y, 1);
		}
		break;
	case 4:
//		return FALSE;
		
		if(!TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 4))
			return FALSE;
		if(!TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1))
			return FALSE;
		aux = (BYTE*)lpBits + m_Size;
		for(y = 0; y < GetImageHeight(); y++)
		{
			aux -= WIDTHBYTES(GetImageWidth() * GetBitsPerPixel());
			TIFFWriteScanline(tif, aux, y, 1);
		}
		
		break;
	case 8:
		if(!TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8))
			return FALSE;
		if(!TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1))
			return FALSE;
		aux = (BYTE*)lpBits + m_Size;
		for(y = 0; y < GetImageHeight(); y++)
		{
			aux -= WIDTHBYTES(GetImageWidth() * GetBitsPerPixel());
			TIFFWriteScanline(tif, aux, y, 1);
		}
		break;
	case 24:
		{
			if(!TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8))
				return FALSE;
			if(!TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 3))
				return FALSE;
			if(!TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB))
				return FALSE;
			aux = (BYTE*)lpBits + m_Size;
			buf = new BYTE[GetImageWidth() * 3];
			for(y = 0; y < GetImageHeight(); y++)
			{
				UINT cont = 0;
				aux -= WIDTHBYTES(Cols() * GetBitsPerPixel());
				while(cont < GetImageWidth() * 3)
				{
					buf[cont    ] = aux[cont + 2];
					buf[cont + 1] = aux[cont + 1];
					buf[cont + 2] = aux[cont    ];
					cont += 3;
				}
				TIFFWriteScanline(tif, buf, y, 3);
				TIFFWriteScanline(tif, buf, y, 2);
				TIFFWriteScanline(tif, buf, y, 1);
			}
			delete []buf;
			break;
		}
	}

	TIFFFlush(tif);
	TIFFClose(tif);
//	SetModifiedFlag(FALSE);     // start off with unmodified
//	EndWaitCursor();
	return TRUE;
}
*/

/*
// Grava um arquivo do tipo GIF (.gif) como nome indicado (caminho completo)
BOOL CDIB::SaveGIFFromFile(PSTR lpszPathName)			
{
	FBM image;
	FILE *file;
	int res;

	// inicializa a estrutura fbm
	image.cm = NULL;
	image.bm = NULL;
	
	// converte a imagem para a estrutura fbm
	ConverterBMPParaGIF(&image);
	
	// abre o arquivo para gravar a imagem
	file = fopen(lpszPathName,"wb");
	if( file == (FILE *)0 ) 
	{
		TRACE("error: could not openGIF output file\n");
		AfxMessageBox("Erro ao abrir arquivo de sada.");
		return FALSE;
	}
	
	// grava a imagem em formato gif
	res = write_gif ((::FBM*)&image, file);

	// libera a memoria da estrutura fbm
	free_fbm(&image);

	return res;
}
*/

BOOL CDIB::SaveTIFF(int fdFile)
{
	return TRUE;
}

// Reinicializa a estrutura do bitmap
BOOL CDIB::ClearDIB()
{
	if (m_pImage != NULL)
		free(m_pImage);

	if (m_palDIB != NULL)
		delete m_palDIB;

	m_pImage = NULL;
	m_nImage = -1;
	m_palDIB = NULL;

	return TRUE;
}

/*
Prepara o DIB apartir das informacoes presentes no cabecalho.
*/
void CDIB::InitDIBData()
{
	PBITMAPINFOHEADER lpbmi;  // pointer to a Win 3.0-style DIB
	PBITMAPCOREHEADER lpbmc;  // pointer to an other-style DIB

	if (m_palDIB != NULL)
		delete m_palDIB;
	m_palDIB = NULL;

	if (m_pImage == NULL)
		return;

	m_pBits = (BYTE*)FindDIBBits();

	m_Width	 = DIBWidth();
	m_Height = DIBHeight();

	m_BitsPPixel = DIBBitsPerPixel();
	m_XPxls = DIB_X_Pxls();
	m_YPxls = DIB_Y_Pxls();

	// initialize m_BinWidth for binary images
	if(IS_WIN30_DIB(m_pImage))
		m_WidthBytes = CALC_WIDTHBYTES( (((PBITMAPINFOHEADER)m_pImage)->biWidth)*
					     		((DWORD)((PBITMAPINFOHEADER)m_pImage)->biBitCount));

	// point to the header (whether old or Win 3.0 
	lpbmi = (PBITMAPINFOHEADER)m_pImage;
	lpbmc = (PBITMAPCOREHEADER)m_pImage;
	
	// Create copy of palette
	m_palDIB = new CPalette;
	if (m_palDIB == NULL)
	{
		printf("erro: nao foi possivel criar CPalette");
		
		// we must be really low on memory
		if (m_pImage)
			free(m_pImage);

		m_pImage = NULL;
		m_nImage = -1;
		m_pBits = NULL;

		return;
	}

	if (CreateDIBPalette() == FALSE)
	{
		// Erro na criacao da paleta -> DIB may not have a palette
		delete m_palDIB;
		m_palDIB = NULL;
		return;
	}

	// determine if first line is inferior or superior
	m_InitSCorner = FALSE;
	if (IS_WIN30_DIB(m_pImage))
		if(lpbmi->biHeight < 0)
		{
			if(!ConvertInitCorner())
				m_InitSCorner = TRUE;
		}
}

BOOL CDIB::ConvertInitCorner()
{
	CDIB NewDib;
	int y, lin;
	PBITMAPINFOHEADER lpbmi; // = (LPBITMAPINFOHEADER)lpImage;

	lpbmi = (PBITMAPINFOHEADER)m_pImage;

	PSTR copy = new char[abs(lpbmi->biHeight) * m_WidthBytes];
	if(copy == NULL)
		return FALSE;

	memcpy(copy, m_pBits, abs(lpbmi->biHeight) * m_WidthBytes);
	lin = abs(lpbmi->biHeight)-1;
	for(y = 0; y < abs(lpbmi->biHeight); y++)
	{
		memcpy(m_pBits + (y * m_WidthBytes), copy + (lin * m_WidthBytes), m_WidthBytes);
		lin --;
	}
	delete copy;
	lpbmi->biHeight = abs(lpbmi->biHeight);
	return TRUE;
}

// Retorna falso no caso de insucesso. Que podem ser:
// m_pImage == NULL ou CreatePallete() nao conseguiu criar paleta. 
BOOL CDIB::CreateDIBPalette()
{
	PLOGPALETTE		pPal;				// pointer to a logical palette
	WORD			wNumColors;         // number of colors in color table
	PBITMAPINFO		lpbmi;				// pointer to BITMAPINFO structure (Win3.0)
	PBITMAPCOREINFO	lpbmc;				// pointer to BITMAPCOREINFO structure (old)
	BOOL			bWinStyleDIB;       // flag which signifies whether this is a Win3.0 DIB
	BOOL			bResult = FALSE;

	// if handle to DIB is invalid, return FALSE 
	if (m_pImage == NULL)
	  return FALSE;

	if(m_palDIB != NULL)
	{
		delete m_palDIB;
		m_palDIB = NULL;
	}
	m_palDIB = new CPalette;

	lpbmi = (PBITMAPINFO)m_pImage;			// get pointer to BITMAPINFO (Win 3.0) 
	lpbmc = (PBITMAPCOREINFO)m_pImage;		// get pointer to BITMAPCOREINFO (old 1.x) 
	wNumColors = DIBNumColors();			// get the number of colors in the DIB 

	if (wNumColors != 0)
	{
		// allocate memory block for logical palette 
		pPal = (PLOGPALETTE) malloc (sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)	* wNumColors);
		
		// set version and number of palette entries 
		pPal->palVersion = PALVERSION;
		pPal->palNumEntries = (WORD)wNumColors;
		bWinStyleDIB = IS_WIN30_DIB(m_pImage);	// is this a Win 3.0 DIB? 
		for (int i = 0; i < (int)wNumColors; i++)
		{
			if (bWinStyleDIB)
			{
				pPal->palPalEntry[i].peRed   = lpbmi->bmiColors[i].rgbRed;
				pPal->palPalEntry[i].peGreen = lpbmi->bmiColors[i].rgbGreen;
				pPal->palPalEntry[i].peBlue  = lpbmi->bmiColors[i].rgbBlue;
				pPal->palPalEntry[i].peFlags = 0;
			}
			else
			{
				pPal->palPalEntry[i].peRed   = lpbmc->bmciColors[i].rgbtRed;
				pPal->palPalEntry[i].peGreen = lpbmc->bmciColors[i].rgbtGreen;
				pPal->palPalEntry[i].peBlue  = lpbmc->bmciColors[i].rgbtBlue;
				pPal->palPalEntry[i].peFlags = 0;
			}
		}
		// create the palette and get handle to it 
		bResult = m_palDIB->CreatePalette(pPal);
		free(pPal);
	}
	return bResult;
}

WORD CDIB::DIBNumColors()
{
	WORD wBitCount;  // DIB bit count
	//  If this is a Windows-style DIB, the number of colors in the
	//  color table can be less than the number of bits per pixel
	//  allows for (i.e. lpbi->biClrUsed can be set to some value).
	//  If this is the case, return the appropriate value.

	ASSERT(m_pImage);
	if (IS_WIN30_DIB(m_pImage))
	{
		DWORD dwClrUsed;
		dwClrUsed = ((PBITMAPINFOHEADER)m_pImage)->biClrUsed;
		if (dwClrUsed != 0)
			return (WORD)dwClrUsed;
	}

	//  Calculate the number of colors in the color table based on
	//  the number of bits per pixel for the DIB.
	if (IS_WIN30_DIB(m_pImage))
		wBitCount = ((PBITMAPINFOHEADER)m_pImage)->biBitCount;
	else
		wBitCount = ((PBITMAPCOREHEADER)m_pImage)->bcBitCount;
	// return number of colors based on bits per pixel 
	switch (wBitCount)
	{
		case 1:
			return 2;
		case 4:
			return 16;
		case 8:
			return 256;
		default:
			return 0;
	}
}

/*
	Mostra no console o infoheader
*/
void CDIB::MostrarInfoHeader()
{

	PBITMAPINFOHEADER pInfoHeader;
	pInfoHeader = (PBITMAPINFOHEADER) m_pImage;
	printf("Mostrando o info header:\n");

	printf("biSize = %d\n", pInfoHeader->biSize);
	printf("biWidth = %d\n", pInfoHeader->biWidth); /* %ld */
	printf("biHeight = %d\n", pInfoHeader->biHeight); /* %ld */
	printf("biPlanes = %d\n", pInfoHeader->biPlanes);
	printf("biBitCount = %d\n", pInfoHeader->biBitCount);
	printf("biCompression = %d\n", pInfoHeader->biCompression);
	printf("biSizeImage = %d\n", pInfoHeader->biSizeImage);
	printf("biXPelsPerMeter = %d\n", pInfoHeader->biXPelsPerMeter); /* %ld */
	printf("biYPelsPerMeter = %d\n", pInfoHeader->biYPelsPerMeter); /* %ld */
	printf("biClrUsed = %d\n", pInfoHeader->biClrUsed);
	printf("biClrImportant = %d\n", pInfoHeader->biClrImportant);
}

/*
BOOL CDIB::LoadJPG(FILE *arquivo)
{
	// JPG
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerro;
    	long linhaBytesJPG;					// Bytes gastos para uma linha, incluindo profundidade
	JSAMPARRAY buffer;
    	
	// Cria a estrutura utilizada na descompressao da imagem 
	cinfo.err = jpeg_std_error(&jerro);
    	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, arquivo);
    
	// Le cabecalho da imagem
    	jpeg_read_header(&cinfo, TRUE);

 	cinfo.quantize_colors = FALSE;
    	jpeg_start_decompress(&cinfo);
    	
	// Aloca a memoria e cria paleta se necessario.
	CreateImage(cinfo.output_width, cinfo.output_height, cinfo.output_components * 8);
	
	// Buffer que recebe uma linha ja descomprimida da imagem jpeg 
    	linhaBytesJPG = cinfo.output_width * cinfo.output_components;
    	buffer = (*cinfo.mem->alloc_sarray)
        	((j_common_ptr) &cinfo, JPOOL_IMAGE, linhaBytesJPG, 1);

	
	// Numero de bits a serem adicionados no final de cada linha do BMP. 
	//(bytes por col sao mult. de 4)
	
	//lixoBytes	linhaBytesJPG % 4
	//0		0
	//1		3	
	//2		2
	//3	 	1	
	
	int lixoBytes = (4 - (linhaBytesJPG % 4))  % 4;

	int linhaBytesBMP = linhaBytesJPG * sizeof(unsigned char) + lixoBytes;
	char * posDestino = FindDIBBits() + (cinfo.output_height - 1) * (linhaBytesJPG + lixoBytes); 
	
	// Transposicao vetor jpg -> bmp. Linha por linha, de baixo para cima do DIB 
    	while(cinfo.output_scanline < cinfo.output_height)
	{
		jpeg_read_scanlines(&cinfo, buffer, 1);
	
//		if ((cinfo.output_scanline < cinfo.output_height - 10) && (cinfo.output_scanline > 10))		
		memcpy(posDestino, buffer[0], linhaBytesBMP);
		
		// Observe que a escrita no DIB eh de baixo para cima
		posDestino -= linhaBytesBMP;
	}

	// Troca de RGB para BGR. JPG tem formato RGB, em DIB eh BGR. 
	if (GetBitsPerPixel() == 24)
	{		
		// posInicio: posicao do primeiro pixel 	= 	 m_pImage + PaletteSize() + sizeof(BITMAPINFOHEADER)
		char *posInicio = FindDIBBits();	
		char *p, tmp;
		int col = 0;
		
		for (p = posInicio; p < posInicio + linhaBytesJPG * cinfo.output_height - 3; p+=3)
		{
				// Podem existir bytes vazios no final de cada linha do BMP (colunas mult de 4)
				if (col >= linhaBytesJPG)
				{
					col = 0;
					p+= lixoBytes;
				}

				tmp = *(p + 2);
				*(p + 2) = *p;
				*p = tmp;	
				
				col += 3;
		}
	}

	// Destroi as estruturas alocadas para o processo de leitura
    	(void) jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	return TRUE;
}
*/

BOOL CDIB::LoadBMP(FILE* fdFile)
{
	BITMAPFILEHEADER bmfHeader;
	// Go read the DIB file header and check if it's valid.
	if (fread((PSTR)&bmfHeader,1,sizeof(bmfHeader),fdFile) != sizeof(bmfHeader))
		return FALSE;
	
	if (bmfHeader.bfType != DIB_HEADER_MARKER())
	{
		printf("LoadBMP: arquivo bmp nao possui marca BM\n");
		return FALSE;
	}
 
	// Allocate memory for DIB
	m_nImage = bmfHeader.bfSize - sizeof(BITMAPFILEHEADER);

	m_pImage = (PSTR)malloc(m_nImage);
	if (m_pImage == NULL)
	{
		m_pImage = NULL;
		m_nImage = -1;
		fprintf(stderr,"LoadBMP: Erro ao alocar memoria!\n");
		return FALSE;
	}
	
	// Go read the bits.
	if (fread(m_pImage,1,m_nImage,fdFile) != (size_t)m_nImage)
	{
		if (m_pImage != NULL)
			free(m_pImage);
		m_pImage = NULL;
		m_nImage = -1;
		fprintf(stderr,"LoadBMP: Erro ao carregar arquivo na memoria!\n");
		return FALSE;
	}
	
	return TRUE;
}


BOOL CDIB::SaveBMP(FILE* fdFile)
{
	BITMAPFILEHEADER bmfHeader; // Header for Bitmap file
	PBITMAPINFOHEADER pbmiHeader;   // Pointer to DIB info structure
	DWORD dwDIBSize;

	if (m_pImage == NULL)
		return FALSE;

	// Get a pointer to the DIB memory, the first of which contains a BITMAPINFO structure
	pbmiHeader = (PBITMAPINFOHEADER)m_pImage;
	if (pbmiHeader == NULL)
		return FALSE;

	if (!IS_WIN30_DIB(pbmiHeader))
		return FALSE;       // It's an other-style DIB (save not supported)

	// Fill in the fields of the file header
	// Fill in file type (first 2 bytes must be "BM" for a bitmap) 
	bmfHeader.bfType = DIB_HEADER_MARKER();  // "BM"

	// Calculating the size of the DIB is a bit tricky (if we want to
	// do it right).  The easiest way to do this is to call GlobalSize()
	// on our global handle, but since the size of our global memory may have
	// been padded a few bytes, we may end up writing out a few too
	// many bytes to the file (which may cause problems with some apps).
	//
	// So, instead let's calculate the size manually (if we can)
	//
	// First, find size of header plus size of color table.  Since the
	// first DWORD in both BITMAPINFOHEADER and BITMAPCOREHEADER conains
	// the size of the structure, let's use this.
	dwDIBSize = *(DWORD*)pbmiHeader;  // Partial Calculation
		
	DWORD dwBmBitsSize;  // Size of Bitmap Bits only
	// Now calculate the size of the image
	if ((pbmiHeader->biCompression == BI_RLE8) || (pbmiHeader->biCompression == BI_RLE4))
	{
		// It's an RLE bitmap, we can't calculate size, so trust the
		// biSizeImage field
		dwBmBitsSize = pbmiHeader->biSizeImage;
	}
	else
	{
		// It's not RLE, so size is Width (DWORD aligned) * Height
		dwBmBitsSize = CALC_WIDTHBYTES((pbmiHeader->biWidth)*((DWORD)pbmiHeader->biBitCount)) * pbmiHeader->biHeight;
		// Now, since we have calculated the correct size, why don't we
		// fill in the biSizeImage field (this will fix any .BMP files which
		// have this field incorrect).
		pbmiHeader->biSizeImage = dwBmBitsSize;
	}

	// Calculate the file size by adding the DIB size to sizeof(BITMAPFILEHEADER)
	bmfHeader.bfSize = dwDIBSize + PaletteSize() + dwBmBitsSize + sizeof(BITMAPFILEHEADER);
	bmfHeader.bfReserved1 = 0;
	bmfHeader.bfReserved2 = 0;

	// Now, calculate the offset the actual bitmap bits will be in
	// the file -- It's the Bitmap file header plus the DIB header,
	// plus the size of the color table.
	bmfHeader.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER) + pbmiHeader->biSize + PaletteSize();
	
	// Write the file header
	if (fwrite((PSTR)&bmfHeader,1,sizeof(BITMAPFILEHEADER),fdFile) != (size_t)sizeof(BITMAPFILEHEADER))
	{
		fprintf(stderr,"SaveBMP: Erro ao gravar BITMAPFILEHEADER!\n");
		return FALSE;
	}

	// Write the DIB header and the bits
	if (fwrite(pbmiHeader,1,dwDIBSize,fdFile) != (size_t)dwDIBSize)
	{
		fprintf(stderr,"SaveBMP: Erro ao gravar Bitmap!\n");
		return FALSE;
	}

	if ( (PaletteSize() > 0 ) )
	{
		// Write the DIB header and the bits
		if (fwrite(&(m_palDIB->m_pPalette->palPalEntry[0]),1, PaletteSize(),fdFile) != (size_t)PaletteSize())
		{
			fprintf(stderr,"SaveBMP: Erro ao gravar Bitmap!\n");
			return FALSE;
		}
	}

	// Write the bitmap by itself
	if (fwrite(m_pBits,1, m_nImage - sizeof(BITMAPINFOHEADER) - PaletteSize(),fdFile) != (size_t)(m_nImage - sizeof(BITMAPINFOHEADER)- PaletteSize()))
	{
		fprintf(stderr,"SaveBMP: Erro ao gravar Bitmap!\n");
		return FALSE;
	}
	
	return TRUE;
}

void CDIB::ReplaceImage(PSTR pImage,int nImage)
{
	if (m_pImage != NULL)
		free(m_pImage);
	
	m_pImage = pImage;
	m_nImage = nImage;

	InitDIBData();
}

PSTR CDIB::CopyImage(int& nImage)
{
	PSTR pCopy;

	ASSERT(m_pImage);
	
	pCopy = (PSTR)malloc(m_nImage);
	if (pCopy != NULL)
	{
		nImage = m_nImage;
		memcpy(pCopy, m_pImage, m_nImage);
	}
	else
		nImage = -1;

	return pCopy;
}

void	CDIB::CreateImage(UINT Width, UINT Height, WORD Bits)
{
	int	MemSize;
	BYTE	cor[16] = { 0,17,34,51,68,85,102,119,136,153,170,187,204,221,238,255 };

	if (m_pImage != NULL)
		delete m_pImage;

	m_pImage = NULL;
	m_nImage = -1;

	PBITMAPINFOHEADER lpTemp;
	int x;
	DWORD WidthB = CALC_WIDTHBYTES(Width * Bits);

	switch(Bits)
	{
	case 1:
		MemSize = sizeof(BITMAPINFOHEADER) + (  2 * sizeof(PALETTEENTRY)) + (WidthB * Height);
		break;
	case 4:
		MemSize = sizeof(BITMAPINFOHEADER) + ( 16 * sizeof(PALETTEENTRY)) + (WidthB * Height);
		break;
	case 8:
		MemSize = sizeof(BITMAPINFOHEADER) + (256 * sizeof(PALETTEENTRY)) + (WidthB * Height);
		break;
	case 24:
		MemSize = sizeof(BITMAPINFOHEADER) + (WidthB * Height);
		break;
	default:
		return;
	}

	m_pImage = (PSTR)malloc(MemSize);
	if(m_pImage == NULL)
		return;

	m_nImage = MemSize;
	for(x = 0; x < MemSize; x++)
		m_pImage[x] = (char)0xFF;

	lpTemp = (PBITMAPINFOHEADER)m_pImage;
	lpTemp->biSize = sizeof(BITMAPINFOHEADER);
	lpTemp->biWidth = Width;
	lpTemp->biHeight = Height;
	lpTemp->biPlanes = 1;
	lpTemp->biCompression = BI_RGB;
	lpTemp->biSizeImage = WidthB * Height;
	lpTemp->biXPelsPerMeter = 0;
	lpTemp->biYPelsPerMeter = 0;
	lpTemp->biClrUsed = 0;
	lpTemp->biClrImportant = 0;
	switch(Bits)
	{
	case 1:
		lpTemp->biBitCount = 1;
		break;
	case 4:
		lpTemp->biBitCount = 4;
		break;
	case 8:
		lpTemp->biBitCount = 8;
		break;
	case 24:
		lpTemp->biBitCount = 24;
	}
	switch(Bits) // inicializa a palleta
	{
	case 1:
		((PBITMAPINFO)m_pImage)->bmiColors[0].rgbBlue		= 0;
		((PBITMAPINFO)m_pImage)->bmiColors[0].rgbRed		= 0;
		((PBITMAPINFO)m_pImage)->bmiColors[0].rgbGreen		= 0;
		((PBITMAPINFO)m_pImage)->bmiColors[0].rgbReserved	= PC_NOCOLLAPSE;
		((PBITMAPINFO)m_pImage)->bmiColors[1].rgbBlue		= 255;
		((PBITMAPINFO)m_pImage)->bmiColors[1].rgbRed		= 255;
		((PBITMAPINFO)m_pImage)->bmiColors[1].rgbGreen		= 255;
		((PBITMAPINFO)m_pImage)->bmiColors[1].rgbReserved	= PC_NOCOLLAPSE;
		break;
	case 4:
		for (x = 0; x < 16; x++)
		{
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbBlue		= cor[x];
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbRed		= cor[x];
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbGreen		= cor[x];
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbReserved = PC_NOCOLLAPSE;
		}
		break;
	case 8:
		for (x = 0; x < 256; x++)
		{
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbBlue		= (BYTE)x;
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbRed		= (BYTE)x;
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbGreen		= (BYTE)x;
			((PBITMAPINFO)m_pImage)->bmiColors[x].rgbReserved = PC_NOCOLLAPSE;
		}
		break;
	}
	InitDIBData();
}

void CDIB::ClearImage(void)
{
	UINT x;
	UINT y;
	switch(m_BitsPPixel)
	{
	case 1:
		for(x=0; x < m_Width; x++)
			for (y=0; y < m_Height; y++)
				SetPixelBW(x, y, 1);
		break;
	case 4:
		for(x=0; x < m_Width; x++)
			for (y=0; y < m_Height; y++)
				SetPixelG4(x, y, 15);
		break;
	case 8:
		for(x=0; x < m_Width; x++)
			for (y=0; y < m_Height; y++)
				SetPixelG8(x, y, 255);
		break;
	case 24:
		for(x=0; x < m_Width; x++)
			for (y=0; y < m_Height; y++)
				SetPixelTC(x, y, RGB(255,255,255));
		break;
	}

}

