#ifndef _DIB_H_
#define _DIB_H_

#include <stdio.h>
#include <stdlib.h>

#include "defs.h"
#include "palette.h"
// DIB.h : header file
//

/*
#ifdef __cplusplus
extern "C" {
    #include <jpeglib.h>
}
#endif
#include <jconfig.h>
#include <jpeglib.h> 
#include <jmorecfg.h>
#include <jerror.h>
*/

// Otimizacao para Trabalho com BW (DAVID)
const BYTE tab[8] = {0x7F, 0xBF,  0xDF, 0xEF, 0xF7, 0xFB, 0xFD, 0xFE};

#define BI_RGB        0L
#define BI_RLE8       1L
#define BI_RLE4       2L
#define BI_BITFIELDS  3L

/////////////////////////////////////////////////////////////////////////////
// CDIB command target

/* Bitmap Header Definition */
#pragma pack(1)
typedef struct tagBITMAP
{
	LONG        bmType;
	LONG        bmWidth;
	LONG        bmHeight;
	LONG        bmWidthBytes;
	WORD        bmPlanes;
	WORD        bmBitsPixel;
	PVOID       bmBits;
} BITMAP, *PBITMAP;

#pragma pack(1)
typedef struct tagBITMAPFILEHEADER {
        WORD    bfType;
        DWORD   bfSize;
        WORD    bfReserved1;
        WORD    bfReserved2;
        DWORD   bfOffBits;
} BITMAPFILEHEADER, *PBITMAPFILEHEADER;

#pragma pack(1)
typedef struct tagRGBTRIPLE {
        BYTE    rgbtBlue;
        BYTE    rgbtGreen;
        BYTE    rgbtRed;
} RGBTRIPLE;

#pragma pack(1)
typedef struct tagRGBQUAD
{
	BYTE    rgbBlue;
	BYTE    rgbGreen;
	BYTE    rgbRed;
	BYTE    rgbReserved;
} RGBQUAD;
typedef RGBQUAD *PRGBQUAD;

#pragma pack(1)
typedef struct tagBITMAPCOREHEADER
{
	DWORD   bcSize;                 /* used to get to color table */
	WORD    bcWidth;
	WORD    bcHeight;
	WORD    bcPlanes;
	WORD    bcBitCount;
} BITMAPCOREHEADER, *PBITMAPCOREHEADER;

#pragma pack(1)
typedef struct tagBITMAPINFOHEADER
{
	DWORD      biSize;
	LONG       biWidth;
	LONG       biHeight;
	WORD       biPlanes;
	WORD       biBitCount;
	DWORD      biCompression;
	DWORD      biSizeImage;
	LONG       biXPelsPerMeter;
	LONG       biYPelsPerMeter;
	DWORD      biClrUsed;
	DWORD      biClrImportant;
} BITMAPINFOHEADER, *PBITMAPINFOHEADER;

#pragma pack(1)
typedef struct tagBITMAPINFO
{
    BITMAPINFOHEADER    bmiHeader;
    RGBQUAD             bmiColors[1];
} BITMAPINFO, *PBITMAPINFO;

#pragma pack(1)
typedef struct tagBITMAPCOREINFO {
    BITMAPCOREHEADER    bmciHeader;
    RGBTRIPLE           bmciColors[1];
} BITMAPCOREINFO, *PBITMAPCOREINFO;

// Classe CDIB
class CDIB
{
	CDIB		operator +  (CDIB &op);
	CDIB		operator +  (CDIB *op);
	CDIB		operator -  (CDIB &op);
	CDIB		operator -  (CDIB *op);
	CDIB		operator =  (CDIB &op);
	CDIB		operator =  (CDIB *op);
	BOOL		operator != (CDIB &op);
	BOOL		operator != (CDIB *op);
	BOOL		operator == (CDIB &op);
	BOOL		operator == (CDIB *op);
// Attributes
protected:

			char*  			m_pImage;			// ponteiro para o bitmap
			int				m_nImage;
			BYTE*			m_pBits;			// ponteiro para os bits do bitmap
			CPalette*		m_palDIB;			// Paleta
			UINT			m_Height;			// altura do bitmap
			UINT			m_Width;			// largura do bitmap
			UINT			m_WidthBytes;			// numero de bytes por linha do bitmap
			WORD			m_BitsPPixel;			// numero de bits por pixel

			BOOL			m_InitSCorner;		// qual a primeira linha do Bitmap : V - superior ; F - inferior
			UINT			m_XPxls;			// numero de pixels por metro na vertical
			UINT			m_YPxls;			// numero de pixels por metro na horizontal

// Operations
public:

// Implementation
public:
	CDIB();				// protected constructor used by dynamic creation
	CDIB(CDIB &New);	// Construtor da classe
	CDIB(CDIB *New);	// Construtor da classe
	~CDIB();			// Destrutor da classe

			PSTR 		CopyImage				(int& nImage);
			void 		ReplaceImage			(PSTR pImage,int nImage);
			void 		CreateImage				(UINT Width, UINT Height, WORD Bits);

public:
	inline	BOOL		IS_WIN30_DIB			( void*) const;				// Verifica a versao do bitmap, retorna verdade para bitmap V3, falso em contrario
	inline	WORD		DIB_HEADER_MARKER		();							// Constroi o header marker para bitmap: "BM"
	inline	UINT		CALC_WIDTHBYTES			(DWORD);

	inline	void		SetDIB_Y_Pxls			( UINT);					// seta o numero de pixels por metro na vertical
	inline	void		SetDIB_X_Pxls			( UINT);					// seta o numero de pixels por metro na horizontal
	inline	UINT		Y_Pxls					();							// retorna o numero de pixels por metro na vertical
	inline	UINT		X_Pxls					();							// retorna o numero de pixels por metro na horizontal

	inline	WORD		PaletteSize				();							// retorna o tamnho da paleta
	inline	int			GetNearestPaletteIndex  (DWORD ) const;
	inline 	BOOL		GetPaletteEntry			(WORD nIndex,PALETTEENTRY& PaletteColor) const;
	inline	BOOL		SetPaletteEntry			(WORD nIndex,PALETTEENTRY PaletteColor);

			void		ClearImage				(void);						// limpa o bitmap
			void		ClearAttributes			(void);

	inline  PSTR		GetImage				() const;					// retorna um ponteiro para o bitmap
	inline	BYTE*		GetImageBits			() const;					// retorna um ponteiro para os bits do bitmap
	inline	UINT		GetImageHeight			();							// retorna o numero de linhas do bitmap
	inline	UINT		GetImageWidth			();							// retorna o numero de colunas do bitmap
	inline	DWORD		GetWidthBytes			()							// numero de bytes por linha do bitmap
												{ return m_WidthBytes; };
	inline	WORD		GetBitsPerPixel			() const;					// retorna o numero de bits usados para cada pixel

	inline	UINT		GetPixel				( UINT, UINT) const;		// retorna o valor RGB do pixel
	inline	BOOL		GetPixelBW				( UINT, UINT);			// retorna o valor do pixel (preto e branco)
	inline	BYTE		GetPixelG4				( UINT, UINT);			// retorna o valor do pixel (16 tons de cinza)
	inline	BYTE		GetPixelG8				( UINT, UINT);			// retorna o valor do pixel (256 tons de cinza)
	inline	COLORREF	GetPixelTC				( UINT, UINT);			// retorna o valor do pixel (true color)
	inline	BYTE		GetPixelTCR				( UINT, UINT);			// retorna o valor vermelho do pixel (true color)
	inline	BYTE		GetPixelTCG				( UINT, UINT);			// retorna o valor verde do pixel (true color)
	inline	BYTE		GetPixelTCB				( UINT, UINT);			// retorna o valor azul do pixel (true color)

	inline	void		SetPixel				( UINT, UINT, COLORREF);	// escreve um pixel, valor RGB
	inline	void		SetPixelBW				( UINT, UINT, BOOL);		// escreve um pixel (preto e branco)
	inline	void		SetPixelG4				( UINT, UINT, BYTE);		// escreve um pixel (16 tons de cinza)
	inline	void		SetPixelG8				( UINT, UINT, BYTE);		// escreve um pixel (256 tons de cinza)
	inline	void		SetPixelTC				( UINT, UINT, COLORREF);	// escreve um pixel (true color)
	inline	void		SetPixelTCR				( UINT, UINT, BYTE);		// escreve o componente vermelho de um pixel (true color)
	inline	void		SetPixelTCG				( UINT, UINT, BYTE);		// escreve o componente verde de um pixel (true color)
	inline	void		SetPixelTCB				( UINT, UINT, BYTE);		// escreve o componente azul de um pixel (true color)

	inline	BOOL		IsPaletted				() const;					// verifica se existe uma paleta para o bitmap
			BOOL		ClearDIB				();							// Reinicializa a estrutura do bitmap

//			BOOL		SaveFromFile			( PCTSTR);					// Grava o documento com o nome indicado (caminho completo)
//			BOOL		LoadFromFile			( PCTSTR);					// Abre o documento que tem o nome indicado (caminho completo)

			BOOL		AbrirArquivo(char *nomeArq);								// Abre um arquivo de extensao JPG ou BMP.
			BOOL		LoadBMPFromFile			( PCTSTR);					// Carrega um arquivo do tipo BitMap (.bmp) com o nome indicado (caminho completo)
			BOOL		LoadJPGFromFile			( PCTSTR);					// Carrega um arquivo do tipo Jpeg   (.jpg) com o nome indicado (caminho completo)
//			BOOL		LoadTIFFFromFile		( PCTSTR);					// Carrega um arquivo do tipo TIFF   (.tif) com o nome indicado (caminho completo)
//			BOOL		LoadGIFFromFile			( PCTSTR);					// Carrega um arquivo do tipo GIF    (.gif) com o nome indicado (caminho completo)

			BOOL		SaveBMPFromFile			( PCTSTR);					// Salva   um arquivo do tipo BitMap (.bmp) com o nome indicado (caminho completo)
//			BOOL		SaveTIFFFromFile		( PCTSTR);					// Salva   um arquivo do tipo TIFF   (.tif) com o nome indicado (caminho completo)
//			BOOL		SaveGIFFromFile			( PCTSTR);					// Salva   um arquivo do tipo GIF    (.gif) com o nome indicado (caminho completo)

//			BOOL		Convert2BW				();							// converte o bitmap para binario
//			BOOL		Convert2Gray4			();							// converte o bitmap para 16 tons de cinza
//			BOOL		Convert2Gray8			();							// converte o bitmap para 256 tons de cinza
//			BOOL		Convert2RGB				();							// converte o bitmap para RGB (true color)

			BOOL		ConvertInitCorner		();

private:
	inline	UINT		DIB_Y_Pxls				();							// retorna o numero de pixels por metro na vertical
	inline	UINT		DIB_X_Pxls				();							// retorna o numero de pixels por metro na horizontal
	inline	WORD		DIBBitsPerPixel			();							// retorna o numero de bits usados para cada pixel
			WORD		DIBNumColors			();							// retorna o numero de cores usadas no bitmap
	inline	UINT		DIBWidth				();							// retorna a largura do bitmap
	inline	UINT		DIBHeight				();							// retorna a altura do bitmap
	inline	UINT		DIBSize					();							// retorna o tamanho do bitmap

// debugacao
	void			MostrarInfoHeader			();

protected:
			BOOL        LoadJPG					(FILE*);					// Carrega um arquivo Jpeg
			BOOL        LoadBMP					(FILE*);						// Carrega um arquivo de BitMap formato Win3.0

			BOOL        SaveBMP					(FILE*);						// Salva   um arquivo de BitMap formato Win3.0
			BOOL		SaveTIFF				(int);						// Salva   um arquivo do tipo TIFF (.tif)

	inline	PSTR		FindDIBBits				();							// retorna um ponteiro para os bits do bitmap
			BOOL        CreateDIBPalette		();							// cria a paleta do bitmap
			void		InitDIBData				();							// inicializa os dados do bitmap e da classe
			void		AssertValidDib			() const;
};

/////////////////////////////////////////////////////////////////////////////

inline	BOOL CDIB::IS_WIN30_DIB(void* lpbi) const
{
	return ( ((PBITMAPINFOHEADER)lpbi)->biSize == sizeof(BITMAPINFOHEADER) );
}

//Dib Header Marker - used in writing DIBs to files
inline	WORD CDIB::DIB_HEADER_MARKER()
{
	return ((WORD) ('M' << 8) | 'B');
}

inline PSTR CDIB::FindDIBBits()
{
	ASSERT(m_pImage);
	return (m_pImage + *(DWORD*)m_pImage + PaletteSize());
}

inline BYTE* CDIB::GetImageBits() const
{
	return m_pBits;
}

inline PSTR CDIB::GetImage() const
{
	return m_pImage;
}

inline UINT CDIB::DIBWidth()
{
	ASSERT(m_pImage);
	// return the DIB width if it is a Win 3.0 DIB
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biWidth;
	else  // it is an other-style DIB, so return its width
		return ((PBITMAPCOREHEADER)m_pImage)->bcWidth;
}

inline UINT CDIB::DIBHeight()
{
	ASSERT(m_pImage);
	// return the DIB height if it is a Win 3.0 DIB
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biHeight;
	else  // it is an other-style DIB, so return its height
		return ((PBITMAPCOREHEADER)m_pImage)->bcHeight;
}

inline UINT CDIB::DIBSize()
{
	ASSERT(m_pImage);
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biSizeImage;
	else  // it is an other-style DIB, so return its width
		return 0;
}


inline UINT CDIB::DIB_X_Pxls()
{
	ASSERT(m_pImage);
	// return the DIB width if it is a Win 3.0 DIB
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biXPelsPerMeter;
	else  // it is an other-style DIB, so return its width
		return 0;
}

inline UINT CDIB::DIB_Y_Pxls()
{
	ASSERT(m_pImage);
	// return the DIB width if it is a Win 3.0 DIB
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biYPelsPerMeter;
	else  // it is an other-style DIB, so return its width
		return 0;
}

inline UINT CDIB::X_Pxls()
{
	return m_XPxls;
}

inline UINT CDIB::Y_Pxls()
{
	return m_YPxls;
}


inline void CDIB::SetDIB_X_Pxls(UINT Pixels)
{
	ASSERT(m_pImage);
	if (IS_WIN30_DIB(m_pImage))
		((PBITMAPINFOHEADER)m_pImage)->biXPelsPerMeter = Pixels;
	m_XPxls = Pixels;
}

inline void CDIB::SetDIB_Y_Pxls(UINT Pixels)
{
	ASSERT(m_pImage);
	if (IS_WIN30_DIB(m_pImage))
		((PBITMAPINFOHEADER)m_pImage)->biYPelsPerMeter = Pixels;
	m_YPxls = Pixels;
}

inline UINT CDIB::GetImageWidth()
{
	return m_Width;
}

inline UINT CDIB::GetImageHeight()
{
	return m_Height;
}

// Retorna a cor na posicao (col,lin)
inline UINT CDIB::GetPixel(UINT col, UINT lin) const
{
	BYTE*			lpTemp;
	PALETTEENTRY	pe;

	lpTemp = (BYTE*)m_pBits;
	lpTemp += m_WidthBytes *  lin;      //( bmWidthBytes * ( (GetHeight() -1) - y ) ); // Height - 1 - y

	switch(GetBitsPerPixel())
	{
		case  1:
		{
			lpTemp += (LONG) col/8;

			m_palDIB->GetPaletteEntry( ((*lpTemp) >> (7-(col % 8))) & 1, pe );
			return RGB( pe.peRed, pe.peGreen, pe.peBlue );
		}
		case  4:
		{
			lpTemp += (LONG) col/2;

			UINT idx;
			if (col % 2)
				idx = *lpTemp & 0x0F;
			else
				idx = *lpTemp >> 4;

			m_palDIB->GetPaletteEntry( idx, pe );
			return RGB( pe.peRed, pe.peGreen, pe.peBlue );
        }
		case  8:
		{
			lpTemp += (LONG) col;

			m_palDIB->GetPaletteEntry( *lpTemp, pe );
			return RGB( pe.peRed, pe.peGreen, pe.peBlue );
		}
		case 24:
		{
			lpTemp += (LONG) 3 * col;
			return RGB(*(lpTemp+2), *(lpTemp+1), *(lpTemp));
		}
    }
	return (DWORD) 0;
}

inline BOOL CDIB::GetPixelBW(UINT col, UINT lin)
{
	PALETTEENTRY	pe;

	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 1);

	long idx = (lin * m_WidthBytes) + (col / 8);
	m_palDIB->GetPaletteEntry( (UINT) (((m_pBits[idx]) >> (7-(col % 8))) & 1), pe );

	return !!pe.peBlue;
}


inline BYTE CDIB::GetPixelG4(UINT col, UINT lin)
{
	PALETTEENTRY	pe;

	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 4);

	long idx = (lin * m_WidthBytes + col / 2);
	if(col % 2)
		m_palDIB->GetPaletteEntry( (UINT) (m_pBits[idx] & 0x0F), pe );
	else
		m_palDIB->GetPaletteEntry( (UINT) (m_pBits[idx] >>   4), pe );
	return pe.peBlue;
}

inline BYTE CDIB::GetPixelG8(UINT col, UINT lin)
{
	PALETTEENTRY	pe;

	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 8);

	m_palDIB->GetPaletteEntry( (UINT) (m_pBits[m_WidthBytes * lin + col]), pe );

	return pe.peBlue;
}

inline COLORREF CDIB::GetPixelTC(UINT col, UINT lin)
{
	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);

	long idx = m_WidthBytes * lin + col * 3;
	return RGB(m_pBits[idx + 2], m_pBits[idx + 1], m_pBits[idx]);
}

inline BYTE CDIB::GetPixelTCR(UINT col, UINT lin)
{
	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);

	long idx = m_WidthBytes * lin + col * 3;
	return m_pBits[idx + 2];
}

inline BYTE CDIB::GetPixelTCG(UINT col, UINT lin)
{
	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);

	long idx = m_WidthBytes * lin + col * 3;
	return m_pBits[idx + 1];
}

inline BYTE CDIB::GetPixelTCB(UINT col, UINT lin)
{
	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);

	long idx = m_WidthBytes * lin + col * 3;
	return m_pBits[idx];
}

// Troca a cor na posicao (col, lin)
inline void CDIB::SetPixel(UINT col, UINT lin, COLORREF dwColor)
{
	BYTE*			lpTemp;
	PALETTEENTRY	pe;

	lpTemp = (BYTE*)m_pBits;
	lpTemp += m_WidthBytes *  lin;      //( bmWidthBytes * ( (GetHeight() -1) - y ) ); // Height - 1 - y

	switch(GetBitsPerPixel())
	{
		case  1:
        {
			lpTemp += (LONG) col/8;

			m_palDIB->GetPaletteEntry( (UINT) dwColor, pe );

			*lpTemp &= tab[col % 8];
			*lpTemp |= ( (!!pe.peBlue) << (7 - (col % 8)));
			return;
        }
		case  4:
        {
			lpTemp += (LONG) col/2;

			m_palDIB->GetPaletteEntry( (UINT) dwColor, pe );

			BYTE bt;
			bt = *lpTemp;
			if (col % 2)
				bt = (BYTE) ( (bt & 0xF0) |  (pe.peBlue & 0x0F)       );
			else
				bt = (BYTE) ( (bt & 0x0F) | ((pe.peBlue & 0x0F) << 4) );

			*lpTemp = bt;
			return;
        }
		case  8:
        {
			lpTemp += (LONG) col;
			m_palDIB->GetPaletteEntry( (UINT) dwColor, pe );

			*lpTemp = pe.peBlue;
			return;
        }
		case 24:
		{
			lpTemp += (LONG) 3 * col;

			*(lpTemp+2) = GetRValue(dwColor);
			*(lpTemp+1) = GetGValue(dwColor);
			*(lpTemp+0) = GetBValue(dwColor);
			return;
        }
	}
}

inline void CDIB::SetPixelBW(UINT col, UINT lin, BOOL Pixel)
{
	PALETTEENTRY	pe;

	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 1);


	m_palDIB->GetPaletteEntry( (UINT) Pixel, pe );

	long idx = (lin * m_WidthBytes) + (col / 8);
	m_pBits[idx] &= tab[col % 8];
	m_pBits[idx] |= ( (!!pe.peBlue) << (7 - (col % 8)) );
}

inline void CDIB::SetPixelG4(UINT col, UINT lin, BYTE Pixel)
{
	PALETTEENTRY	pe;
	BYTE temp;

	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 4);

	m_palDIB->GetPaletteEntry( (UINT) Pixel, pe );

	temp = m_pBits[lin * m_WidthBytes + col / 2];
	if(col % 2)
		temp = (BYTE) ((temp & 0xF0) | (pe.peBlue &  0x0F));
	else
		temp = (BYTE) ((temp & 0x0F) | (pe.peBlue <<    4));
	m_pBits[lin * m_WidthBytes + col / 2] = temp;
}

inline void CDIB::SetPixelG8(UINT col, UINT lin, BYTE Pixel)
{
	PALETTEENTRY	pe;

//	fprintf(stdout,"width %d, height %d, col %d, lin %d, Pixel %d\n",m_Width,m_Height,col,lin,Pixel);
	ASSERT(m_pBits);
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 8);

	m_palDIB->GetPaletteEntry( (WORD) Pixel, pe );

	m_pBits[m_WidthBytes * lin + col] = pe.peBlue;
}

inline void CDIB::SetPixelTC(UINT col, UINT lin, COLORREF Pixel)
{
	ASSERT(m_pBits);
	long idx;
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);
	idx = m_WidthBytes * lin + col * 3;
	m_pBits[idx + 2] = GetRValue(Pixel);
	m_pBits[idx + 1] = GetGValue(Pixel);
	m_pBits[idx    ] = GetBValue(Pixel);
}

inline void CDIB::SetPixelTCR(UINT col, UINT lin, BYTE Pixel)
{
	ASSERT(m_pBits);
	long idx;
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);
	idx = m_WidthBytes * lin + col * 3;
	m_pBits[idx + 2] = Pixel;
}

inline void CDIB::SetPixelTCG(UINT col, UINT lin, BYTE Pixel)
{
	ASSERT(m_pBits);
	long idx;
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);
	idx = m_WidthBytes * lin + col * 3;
	m_pBits[idx + 1] = Pixel;
}

inline void CDIB::SetPixelTCB(UINT col, UINT lin, BYTE Pixel)
{
	ASSERT(m_pBits);
	long idx;
	ASSERT((unsigned)col >= 0 && (unsigned)col < m_Width);
	ASSERT((unsigned)lin >= 0 && (unsigned)lin < m_Height);
	ASSERT(GetBitsPerPixel() == 24);
	idx = m_WidthBytes * lin + col * 3;
	m_pBits[idx    ] = Pixel;
}


inline WORD CDIB::PaletteSize()
{
	ASSERT(m_pImage);
	// calculate the size required by the palette
	if (IS_WIN30_DIB (m_pImage))
		return (WORD)(DIBNumColors() * sizeof(RGBQUAD));
	else
		return (WORD)(DIBNumColors() * sizeof(RGBTRIPLE));
}

inline WORD CDIB::GetBitsPerPixel() const
{
	return m_BitsPPixel;
}


inline WORD CDIB::DIBBitsPerPixel()
{
	ASSERT(m_pImage);
	if (IS_WIN30_DIB(m_pImage))
		return ((PBITMAPINFOHEADER)m_pImage)->biBitCount;
	else
		return ((PBITMAPCOREHEADER)m_pImage)->bcBitCount;
}

// Retorna o indice da cor mais proxima da paleta
inline int CDIB::GetNearestPaletteIndex( DWORD dwColor ) const
{
	if (!IsPaletted())
		return 0;
	return m_palDIB->GetNearestPaletteIndex( dwColor );
}


// Retorna as entradas da paleta
inline BOOL CDIB::GetPaletteEntry(WORD nIndex,PALETTEENTRY&  PaletteColor) const
{
	if (!IsPaletted())
		return FALSE;

	return m_palDIB->GetPaletteEntry(nIndex,PaletteColor);
}

// Troca entradas da paleta
inline BOOL CDIB::SetPaletteEntry(WORD nIndex,PALETTEENTRY PaletteColor )
{
	if (!IsPaletted())
		return FALSE;

	PaletteColor.peFlags = 0; // by warning avoiding
	
	return m_palDIB->SetPaletteEntry(nIndex,PaletteColor);
}

// Verifica se existe uma Paleta
inline BOOL CDIB::IsPaletted() const
{
	WORD BPPixel;
	if (IS_WIN30_DIB(m_pImage))
		BPPixel = ((PBITMAPINFOHEADER)m_pImage)->biBitCount;
	else
		BPPixel = ((PBITMAPCOREHEADER)m_pImage)->bcBitCount;
	switch(BPPixel)
	{
		case  1:
		case  4:
		case  8:
			return TRUE;
		default:
			return FALSE;
	}
}

inline	UINT CDIB::CALC_WIDTHBYTES(DWORD bits)
{
	return (((bits) + 31) / 32 * 4);
}

#endif // _DIB_H_
