#include <string.h>

#include "equalization.h"

Equalization::~Equalization()
{
	ClearData();
}

Equalization::Equalization(char * sFileName,int bEHE,TypeAlgorithm tpAlgorithm)
{
	m_bEHE        = bEHE;
	m_tpAlgorithm = tpAlgorithm;

	InitData();

	if (sFileName == (const char*)"")
	{
		fprintf(stderr, "NULL parameter for file name\n");
		exit(3);
	}
	
	m_pImgIn = new CDIB();
	if (!m_pImgIn->AbrirArquivo(sFileName))
	{
		fprintf(stderr,"error while opening image file (%s)!\n", sFileName);
		exit(3);
	}

	strcpy(m_sFileName, sFileName);
	
	m_iWidth  = m_pImgIn->GetImageWidth();
	m_iHeight = m_pImgIn->GetImageHeight();
	
	m_iPixels = m_iWidth * m_iHeight;

	ConstructHist();

	m_pImgOut = new CDIB();
	m_pImgOut->CreateImage(m_iWidth, m_iHeight,8);
}

void Equalization::Saving()
{
	char sImgOut[BUFSIZ],*psOut;

	strcpy(sImgOut, m_sFileName);
	psOut=strrchr(sImgOut,'.');
	*psOut = '\0';

	sprintf(sImgOut,"%s_%s_%s.bmp",sImgOut,GetHistogramEqualizationType(),GetAlgorithm());

	fprintf(stdout, "trying to save: '%s'\n", sImgOut);
	m_pImgOut->SaveBMPFromFile(sImgOut);
}

void Equalization::InitData(void)
{
	m_pH      = NULL;
	m_pC      = NULL;
	m_pNL     = NULL;

	m_pPPL    = NULL;
	m_pLList  = NULL;
	m_pPLList = NULL;
	m_pPtList = NULL;

	m_pImgIn  = NULL;
	m_pImgOut = NULL;
}

void Equalization::ClearData(void)
{
	if ( (m_pH      != NULL) ) free(m_pH);
	if ( (m_pC      != NULL) ) free(m_pC);
	if ( (m_pNL     != NULL) ) free(m_pNL);
	
	if ( (m_pPPL    != NULL) ) free(m_pPPL);
	if ( (m_pLList  != NULL) ) free(m_pLList);
	if ( (m_pPLList != NULL) ) free(m_pPLList);
	if ( (m_pPtList != NULL) ) free(m_pPtList);
	

	if ( (m_pImgIn  != NULL) ) delete m_pImgIn ;
	if ( (m_pImgOut != NULL) ) delete m_pImgOut;

	InitData();
}


char* Equalization::GetHistogramEqualizationType(void)
{
	switch(m_bEHE)
	{
	case 0: return "CHE";
	case 1: return "EHE";
	}
	return "";
}

void Equalization::Equalizing()
{
	if (!m_bEHE)
	{
		register int i,j;

		for ( i = 0; i < m_iHeight ; i++ )
		for ( j = 0; j < m_iWidth  ; j++ )
			m_pImgOut->SetPixelG8( j , i , m_pNL[ m_pImgIn->GetPixelG8( j, i ) ] );
	}
	else
	{
		register int p;
		int le;

		le = 0;
		for ( p = 0; p < m_iPixels ; p++ )
		{
			if ( m_pNL[le] < m_pPPL[le] )
			{
				m_pNL[le]++;
				m_pImgOut->SetPixelG8( m_pPtList[p].x, m_pPtList[p].y , (BYTE)le);
			}
			else
			{
				while(m_pNL[++le] != 0);
				p--;
			}
		}
	}
}

void Equalization::ConstructHist(void)
{
	register int i,j,l;

	m_pH  = (int   *)calloc(256,sizeof(int   ));
	m_pC  = (double*)calloc(256,sizeof(double));
	m_pNL = (int   *)calloc(256,sizeof(int   ));


	for ( i = 0 ; i < m_iHeight ; i++ )
	for ( j = 0 ; j < m_iWidth  ; j++ )
	{
		m_pH[ m_pImgIn->GetPixelG8(j,i) ]++;
	}

	if (m_bEHE)
	{
		m_pLList     = (int   *)calloc(256,sizeof(int   ));
		m_pPLList    = (int   *)calloc(256,sizeof(int   ));
		m_pPPL       = (int   *)calloc(256,sizeof(int   ));

		m_pLList [0] = 0;
		m_pPLList[0] = 0;
		for( l = 1 ; l < 256 ; l++ )
		{
			m_pLList [l] = m_pLList[l-1] + m_pH[l-1];
			m_pPLList[l] = m_pLList[l];
		}

		m_pPtList    = (CPoint*)calloc(m_iPixels,sizeof(CPoint));
		for ( i = 0 ; i < m_iHeight ; i++ )
		for ( j = 0 ; j < m_iWidth  ; j++ )
		{
			l = m_pImgIn->GetPixelG8(j,i);
			m_pPtList[m_pPLList[l]].x = j;
			m_pPtList[m_pPLList[l]].y = i;
			m_pPLList[l]++;
		}

		m_pPLList[0] = 0;
		for( l = 1 ; l < 256 ; l++ )
			m_pPLList[l] = m_pLList[l];
	}
}

void Equalization::HistEq(int low,int high)
{
	register int l;
	int N;

	N = 0;
	for ( l = low ; l <= high; l++ )
		N += m_pH[l];

	if (!m_bEHE)
	{
		/* cumumative function computation */
		m_pC[low] = (double) m_pH[low] / N;
		for ( l = low+1 ; l <= high; l++ )
		{
			m_pC[l] = (double) m_pH[l] / N;
			m_pC[l] += m_pC[l-1];
		}

		/* Histogram Equalization */
		for ( l = low ; l <= high ; l++ )
		{
//			m_pNL[l] = (int) ( m_pC[l]* L -1. +0.5 );
			m_pNL[l] = (int) ( low + m_pC[l]*(high-low) +0.5 );
		}
	}
	else
	{
		/* cumumative function computation */
		m_pC[low] = (double) (high-low+1) / N;
		for ( l = low+1 ; l < high; l++ )
		{
			m_pC[l] = (double) (high-low+1) / N;
			m_pC[l] += m_pC[l-1];
		}
		m_pC[high] += m_pC[l-1];


		/* Histogram Equalization */
		int CC = 0;
		for ( l = low ; l < high ; l++ )
		{
			m_pPPL[l] = (int) ( ((l-low+1) * N / (high-low+1) - CC) + 0.5);
			m_pNL [l] = 0;

			CC += m_pPPL[l];
		}
		m_pPPL[high] = N - CC;
		m_pNL [high] = 0;
	}
}
