#include "equalization.h"
#include "HE.h"
#include "BBHE.h"
#include "DSIHE.h"
#include "MMBEBHE.h"
#include "MMBEMHE.h"
#include "BPHEME.h"
#include "RMSHE.h"

int main(int argc, char* argv[])
{
	int bEHE=0;
	int iAlgorithm,r=0;

	TypeAlgorithm tpAlgorithm = AL_NONE;

	if (argc < 4)
	{
		fprintf(stderr, "use %s [file.bmp] [0|1] 1      , for HE      - Histogram Equalization\n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 2      , for BBHE    - Brightness preserving Bi-Histogram equalization\n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 3      , for DSIHE   - Dualistic Sub-Image Histogram Equalization\n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 4      , for MMBEBHE - Minimum Mean Brightness Error Bi-Histogram Equalization\n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 5 [r=3], for MMBEMHE - Minimum Mean Brightness Error Multi-Histogram Equalization\n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 6      , for BPHEME  - Brightness preserving Histogram Equalization with Maximum Entropy \n", argv[0]);
		fprintf(stderr, "use %s [file.bmp] [0|1] 7 [r=2], for RMSHE   - Recursive Mean-Separate Histogram Equalization\n", argv[0]);

		exit(1);
	}

	bEHE = atoi(argv[2]);
	if ( (bEHE != 0) && (bEHE != 1) )
	{
		fprintf(stderr,"the second parameter, i.e., Exact HE, should be 0 or 1!\n");
		exit(2);
	}

	iAlgorithm = atoi(argv[3]);

	     if ( (iAlgorithm == 1) )
		tpAlgorithm = AL_HE;
	else if ( (iAlgorithm == 2) )
		tpAlgorithm = AL_BBHE;
	else if ( (iAlgorithm == 3) )
		tpAlgorithm = AL_DSIHE;
	else if ( (iAlgorithm == 4) )
		tpAlgorithm = AL_MMBEBHE;
	else if ( (iAlgorithm == 5) )
	{
		tpAlgorithm = AL_MMBEMHE;
		r = argc >= 4 ? atoi(argv[3]) : 3;
	}
	else if ( (iAlgorithm == 6) )
		tpAlgorithm = AL_BPHEME;
	else if ( (iAlgorithm == 7) )
	{
		tpAlgorithm = AL_RMSHE;
		r = argc >= 4 ? atoi(argv[3]) : 2;
	}

	Equalization *pEq = NULL;

	switch(iAlgorithm)
	{
	case 1: pEq = new HE     (argv[1],bEHE);   break;
	case 2: pEq = new BBHE   (argv[1],bEHE);   break;
	case 3: pEq = new DSIHE  (argv[1],bEHE);   break;
	case 4: pEq = new MMBEBHE(argv[1],bEHE);   break;
	case 5: pEq = new MMBEMHE(argv[1],bEHE,r); break;
	case 6: pEq = new BPHEME (argv[1],bEHE);   break;
	case 7: pEq = new RMSHE  (argv[1],bEHE,r); break;
	default: fprintf(stderr,"error, no algorithm selected\n"); exit(1);
	}
	// Equaliza
	pEq->Equalizing();

	// Grava o arquivo
	pEq->Saving();

	delete pEq;
	return 0;
}
