#include <stdio.h>

#include "palette.h"

CPalette::CPalette()
{
	m_pPalette = NULL;
}

CPalette::~CPalette()
{
	DestroyPalette();
}

BOOL	CPalette::CreatePalette(PLOGPALETTE pLogPalette)
{
	if (m_pPalette != NULL)
		DestroyPalette();

	// allocate memory block for logical palette
	if ( (m_pPalette = (PLOGPALETTE)new BYTE[sizeof(LOGPALETTE) + sizeof(PALETTEENTRY)	* pLogPalette->palNumEntries]) == NULL)
	{
		fprintf(stderr,"CreatePalette: Falta de memoria para paleta!");
		return false;
	}

	m_pPalette->palVersion = PALVERSION;
	m_pPalette->palNumEntries = (WORD)pLogPalette->palNumEntries;
	if (m_pPalette->palNumEntries != 0)
	{
		for (int i = 0; i < (int)m_pPalette->palNumEntries; i++)
		{
			m_pPalette->palPalEntry[i].peRed   = pLogPalette->palPalEntry[i].peRed   ;
			m_pPalette->palPalEntry[i].peGreen = pLogPalette->palPalEntry[i].peGreen ;
			m_pPalette->palPalEntry[i].peBlue  = pLogPalette->palPalEntry[i].peBlue  ;
			m_pPalette->palPalEntry[i].peFlags = pLogPalette->palPalEntry[i].peFlags ;	
		}
	}

	return TRUE;
}

BOOL	CPalette::GetPaletteEntry(UINT nIndex, PALETTEENTRY& PaletteColor) const
{
	if ( (nIndex < 0) || (nIndex >= m_pPalette->palNumEntries) )
		return FALSE;

	PaletteColor = m_pPalette->palPalEntry[nIndex];

	return TRUE;
}

BOOL	CPalette::SetPaletteEntry(UINT nIndex, PALETTEENTRY PaletteColor)
{
	if ( (nIndex < 0) || (nIndex >= m_pPalette->palNumEntries) )
		return FALSE;

	m_pPalette->palPalEntry[nIndex] = PaletteColor;

	return TRUE;
}

int		CPalette::GetNearestPaletteIndex(COLORREF crColor) const
{
	int i,iPalMenor;
	BYTE r,g,b;

	r = (crColor>>16)&0xff;
	g = (crColor>> 8)&0xff;
	b =  crColor     &0xff;

	if (m_pPalette->palNumEntries == 0)
		return -1;

	double* dist = new double[m_pPalette->palNumEntries];

	for(i=0;i<m_pPalette->palNumEntries;i++)
	{

		dist[i] = (r - m_pPalette->palPalEntry[i].peRed  ) * (r - m_pPalette->palPalEntry[i].peRed  ) +
		          (g - m_pPalette->palPalEntry[i].peGreen) * (r - m_pPalette->palPalEntry[i].peGreen) +
		          (b - m_pPalette->palPalEntry[i].peBlue ) * (r - m_pPalette->palPalEntry[i].peBlue );
	}

	iPalMenor = 0;
	for(i=1;i<m_pPalette->palNumEntries;i++)
	{
		if ( (dist[iPalMenor] > dist[i]) )
			iPalMenor = i;
	}
	delete []dist;

	return iPalMenor;
}

void	CPalette::DestroyPalette()
{
	if (m_pPalette != NULL)
		delete[] m_pPalette;
	m_pPalette = NULL;
}
